function [ M ] = plotSimulation(log, fig, exportStruct )
%PLOTSIMULATION Summary of this function goes here
%   Detailed explanation goes here
%% PLOTTING

%
% Setup
%

% figureName

if nargin < 3 
    exportFigure     = 0;       % True/false
else
    exportFigure     = exportStruct.doExport;
    exportFigureName = exportStruct.name;
    exportTimeSlots  = exportStruct.timeSlots;
    exportDoExtra    = exportStruct.doExtra;
    
    if isfield(exportStruct, 'minBox')
        exportMinBox = exportStruct.minBox;
    else
        exportMinBox = [3 3];
    end
end


% Just a variable namechange
data = log.get('eta').Values.Data;
%static_obstacles = dataStruct.static_obstacles;
N = 1
time = log.get('eta').Values.Time;

stopTime = time(end);

obsData = log.get('Obstacle').Values.Data;
Nobs = size(obsData, 1);



PlotTarget = exportStruct.PlotTarget;

% Start/stop stuff
statusRun = 1;

% Movie container
M = repmat(getframe, 0);

% 
% Figure Information
%

if nargin < 4
    fig = figure(1); 
end
clf; 
hold on;
set(fig, 'Tag', 'agentScreen');
set(fig, 'KeyPressFcn', @keyPress);

fig_axes = get(fig, 'CurrentAxes');
set(fig_axes, 'DrawMode', 'fast');


foundFig = findobj('Tag','agentScreen');

%
% Plot axis
%


allX = data(:,1)';
allY = data(:,2)';
ratio = exportStruct.ratio ;
%ratio = 1.5;    %% EDIT THIS!!! done


xlim([ratio*min([allX -5]) ratio*max(allX)]);
ylim([ratio*min([allY -40]) ratio*max(allY)]);


% Comment in to enable a zero-zero line.
%line(xlim, [0 0], 'Color', 'k');
%line([0 0], ylim, 'Color', 'k');

%
% Setup plot information
%
hold on; axis equal;
xlabel('x [m]');
ylabel('y [m]');

colors = {'blue', 'red', 'green', 'magenta', 'cyan', 'yellow'};

oldPlots = zeros(1,2*N);
oldObsPlots = zeros(1,2*Nobs);
oldBaryPlot = zeros(1,2);

oldExtraPlot = [];


% Set index to stop the drawing
stopIndex = find(time == stopTime)
startIndex = 1;
stepIndex = 10;
currentIndex = startIndex; % set by the continuePlot function
stopIndex_full = stopIndex; % used to reset stop index

% Export-counter
exportCounter = 1;
PlotTargetCounter = 1;
% % Plot obstacles
% % If ther are any obstacles
% 
% if(size(static_obstacles, 1) >= 2)
% 
%     for i = 1:size(static_obstacles, 2)
%        obstacle = static_obstacles(:,i);
% 
%        t = 0:0.01:2*pi;
%        radi = CTRLconfig.obstacle_d;
% 
%        plot(obstacle(1) + radi*cos(t), obstacle(2) + radi*sin(t), 'k');
% 
%     end
% end
target = exportStruct.target;


% Start the plotting
continuePlot();

    function continuePlot
        
        % remove extra plots
        if any(ishandle(oldExtraPlot))
            %oldExtraPlot
            delete(oldExtraPlot);
            oldExtraPlot = [];
        end
        
        for t = startIndex:stepIndex:stopIndex
            currentIndex = t;
            if statusRun == 0
                
                % store info
                startIndex = t;
                
                break;
            end
            
            
            if exportFigure == 0
                %pause(0.01) 
                fprintf('At time: %f \n', time(t));
            end
            

            timeFrame = 1:t;
            
         
            for i = 1:N
                %data
               % xpos = reshape(data(i,1,timeFrame), [length(timeFrame) 1]);
                %ypos = reshape(data(i,2,timeFrame), [length(timeFrame) 1]);
                
                xpos = data(timeFrame, 1);
                ypos = data(timeFrame, 2);

                %
                % Remove the old plot.
                %
                if oldPlots(2*i) == 0
                    disp('First run');
                elseif ~isempty(foundFig)
                    %delete(oldPlots(4*i));
                    %delete(oldPlots(4*i-1));
                    delete(oldPlots);
                end

                oldPlots(4*i-1) = plot(xpos, ypos, [colors{i} '--']);
                %oldPlots(4*i) = plot(xpos(end), ypos(end), [colors{i} 'o']);
                %oldPlot = plot(xpos, ypos, '.');
                 psi = data(t,3);
                 boat = [-0.5 -1; -0.5 1; 0 2; 0.5 1;0.5 -1; -0.5 -1]'*10;

                 R = @(psi)([cos(psi) -sin(psi); sin(psi) cos(psi)]);

         
                 boat = R(psi-pi/2)*boat;
                 oldPlots(4*i-2) = plot(boat(1,:)+xpos(end), boat(2,:)+ypos(end));
                 oldPlots(4*i-3) = patch(boat(1,:)+xpos(end), boat(2,:)+ypos(end), 'y');

                title(sprintf('NSB controller at time: %.2fs', time(t)));

                % Plot last position as a circle
                %plot(xpos(end), ypos(end), [colors{i} 'o']);


            end
            
            %% All obstacles
            
            % number of rows
            
            
            for obs = 1:Nobs
                
                xpos = reshape(obsData(obs,1,timeFrame), [length(timeFrame) 1]);
                ypos = reshape(obsData(obs,2,timeFrame), [length(timeFrame) 1]);
                
                radii = obsData(obs, 3, t); % Current radii
                

                % Remove the old plot.
                %
                if oldObsPlots(2*obs) == 0
                    disp('First run');
                elseif ~isempty(foundFig)
                    delete(oldObsPlots(2*obs));
                    delete(oldObsPlots(2*obs-1));
                end
                
                theta = 0:0.1:2*pi;
                
                oldObsPlots(2*obs-1) = plot(xpos, ypos, [colors{2} '--']);
                oldObsPlots(2*obs) = plot(xpos(end) + radii*cos(theta), ypos(end) + radii*sin(theta), [colors{2} '-']);
            end
          



            drawnow;


            %
            % Export?
            %
            if exportFigure
               % Check timeslot
               if  exportCounter <= length(exportTimeSlots) && time(t) >= exportTimeSlots(exportCounter)
                   
                   % Check if we should plot extras
                   if length(exportDoExtra) >= exportCounter
                      this_doExtra = exportDoExtra(exportCounter);
                   else
                       this_doExtra = 0;
                   end
                   if length(PlotTarget) >= PlotTargetCounter
                      this_doExtraTarget = PlotTarget(PlotTargetCounter)
                      PlotTarget
                   else
                       this_doExtraTarget = 0
                   end
                   if this_doExtraTarget
                       
                       b = plot(target(1),target(2),'*k');
                       a = text(target(1)-45,target(2)+ 25,'Target');
                       
                   end
                  
                   
                   
                   %path = 'export/';
                   
                   
                   % Export full picture
                   exportPlot(fig, sprintf('%s_%d', exportFigureName,exportCounter), 'full');
                   
                   % Export small picture
                   exportPlot(fig, sprintf('%s_%d_small', exportFigureName,exportCounter), 0.5);
                   
                   
                    % Plot extra
                   if this_doExtra
                       plotExtra();
                   end
                   
                   % Export small picture of where the agents are
                   oldAxis = axis;
                   
                   % "zoom" to the current positions
                   xpos = data(currentIndex,1);
                   ypos = data(currentIndex,2);
                   
                   %xpos = [data(currentIndex-10,1) data(currentIndex+10,1)];
                   %ypos = [data(currentIndex-10,2) data(currentIndex+10,2)];
                   %xlim([ratio*min([xpos]-0.1) ratio*max([xpos min(xpos)+2])]);
                   %ylim([ratio*min([ypos]-0.1) ratio*max([ypos min(ypos)+3])]);
                   
                   x = [min(xpos) max(xpos)];
                   y = [min(ypos) max(ypos)];
                   
                   xlim([min([x(1) mean(x)-exportMinBox(1)/2]), ...
                       max([x(2) mean(x)+exportMinBox(1)/2])]);
                   
                   ylim([min([y(1) mean(y)-exportMinBox(2)/2]), ...
                       max([y(2) mean(y)+exportMinBox(2)/2])]);
                   
                   %xlim([min(xpos)-exportMinBox(1)/6 max([xpos min(xpos)+exportMinBox(1)])]);
                   %ylim([min(ypos)-exportMinBox(2)/6 max([ypos min(ypos)+exportMinBox(2)])]);
                   
                   exportPlot(fig, sprintf('%s_%d_small_zoom', exportFigureName,exportCounter), 0.5);
                   exportPlot(fig, sprintf('%s_%d_zoom', exportFigureName,exportCounter), 'full');

                   %hgexport(fig, sprintf('%s%s%g.eps', path, exportFigureName, exportCounter));
                   %input('Press enter', 's')
                   
                   
                   % Restore old axis
                   axis(oldAxis);
                   
                   
                   % delete extra
                   if this_doExtra
                        if any(ishandle(oldExtraPlot))
                            delete(oldExtraPlot);
                            oldExtraPlot = [];
                        end
                   end
                   if this_doExtraTarget
                       if ishandle(a)
                            delete(a)
                            delete(b)
                            a = [];
                            b = [];
                       end
                   end
                   
                   

                   % Increment the counter
                   exportCounter = exportCounter + 1;
                   PlotTargetCounter = PlotTargetCounter +1;
                   
               end

            end % end export
            
            % Testing some movie export
            %oldPos = get(fig, 'Position');            
            %set(fig, 'Position', [oldPos(1:2) 720 609]);     
            M(end+1) = getframe(fig); 
            %set(fig, 'Position', oldPos);
            
        end % End main loop
        
        % done plotting 
        statusRun = 0;
        
    end % end function



function plotExtra
    if any(ishandle(oldExtraPlot))
        delete(oldExtraPlot);
        oldExtraPlot = [];
    end
    
    % Plot a circle around each agent, with the correct d
    
    k = 0:0.01:2*pi;
    
    for agent = 1:N
       %pos = data(:,agent,currentIndex);
       pos = data(currentIndex, 1:2)';
       r = 1;
    
       
       %oldExtraPlot(end+1) = plot(pos(1) + r*cos(k), pos(2) + r*sin(k));
       
       % Plot velocity vector
       v                        = log.get('V_d').Values.Data(currentIndex,:);
       v_circle                 = log.get('V1').Values.Data(currentIndex,:);
       v_tagent                 = log.get('V2').Values.Data(currentIndex,:);
       v_target                 = log.get('V3').Values.Data(currentIndex,:);
       pos_obs                  = log.get('ObstacleToNSB').Values.data(currentIndex,1:2);
       diameter                 = log.get('ObstacleToNSB').Values.data(currentIndex,4);
       threat                   = log.get('ObstacleToNSB').Values.data(currentIndex,5);
       direction                = log.get('ObstacleToNSB').Values.data(currentIndex,6);
       
       
       
       %% scale arrows v1,v2,v3 and vd
       V = [norm(v_circle) norm(v_tagent) norm(v_target) norm(v)];
       [maxValue,vectorMax] = max(V);
       
       if maxValue > 70
           scaleValue = round(V(vectorMax)/70);
           v_circle = v_circle./scaleValue;
           v_tagent = v_tagent./scaleValue;
           v_target = v_target./scaleValue;
           v = v./scaleValue;
           
       end
       
       
       
       
       
       oldExtraPlot = vector_append(oldExtraPlot, plotVectorWithAnnotation(pos, v, 'b--', 'Vd'));
       
       if norm(v_circle) > 0
       oldExtraPlot = vector_append(oldExtraPlot, plotVectorWithAnnotation(pos, v_circle, 'k--', 'V1'));
       end
       if norm(v_tagent) > 0 && norm(v_tagent) ~= norm(v)
       oldExtraPlot = vector_append(oldExtraPlot, plotVectorWithAnnotation(pos, v_tagent, 'k--', 'V2.1'));
       end
       if norm(v_target) > 0 && norm(v_target) ~= norm(v)
       oldExtraPlot = vector_append(oldExtraPlot, plotVectorWithAnnotation(pos, v_target, 'k--', 'V2.2'));
       end
       
       disp(['Obstacle at position: ','[',num2str(pos_obs(1)),' , ', num2str(pos_obs(2)),']'])
       disp(['Radius: ', num2str(diameter)])
       if threat == 1
           disp('Threat: Yes')
       else
           disp('Threat: No')
       end
       if direction ==1
           dir = 'clockWise';
       else
           dir = 'antiClockWise';
       end
       disp(['Direction around :', num2str(dir)])
       % Display information in the commandline
       %fprintf('Agent: %d \n', agent);
       

       
    end
    
    
    
end



function keyPress(src, eventData)
    fprintf('Pressed key: %s \n', eventData.Key);
    
    if eventData.Key == 's'
        statusRun = ~statusRun;
        
        % set correct stop time again (always full)
        stopIndex = stopIndex_full;
        
        % if starting
        if statusRun == 1
            continuePlot();
        else
           plotExtra(); 
        end
        
    end
    
    if eventData.Key == 'r'
        if statusRun == 0
            startIndex = 1;
            statusRun = 1;
            continuePlot();
        end
    end
    
    switch eventData.Key
        case 'd' % step
            if statusRun == 0
                startIndex = currentIndex;
                stopIndex = currentIndex + stepIndex;
                statusRun = 1;
                continuePlot();
                plotExtra();
            end
    end
end




end

function out = vector_append(v, new)
    out = zeros(1, length(new));
    for i = 1:length(new)
        out(i) = new(i);
    end
    
    out = [v out];
end

function handle = plotVector(pos, v, style)
%Plots the vector from the pos
% Does magic
    
    % find vector angle
    angle = atan2(v(2), v(1));
    
    % Get rotation matrix
    R = @(a)( [cos(a) -sin(a); sin(a) cos(a)]);
    
    % Length of vector
    length = norm(v);
    
    % Calculate the vector
    vector = R(angle) * [length; 0];
    
    % Line positions
    lineX = [pos(1) pos(1) + vector(1)];
    lineY = [pos(2) pos(2) + vector(2)];
    
    % Plot the line    
    handle = line(lineX, lineY);
    
    % Set style
    t = regexp(style, '(?<color>\w*)', 'names');
    color = t.color;
    t = regexp(style, '(?<linestyle>\W*)', 'names');
    linestyle = t.linestyle;
    
    set(handle, 'Color', color);
    set(handle, 'LineStyle', linestyle);
    
    
end


function handle = plotVectorWithAnnotation(pos, v, style, theText)
    % ARROW SETUP
    tip = 0.3;
    scale = 7;
    arrow = [0 0;
                -tip -1;
                0 -0.7;
                tip -1;
                0 0;]';   
    
    % find vector angle
    angle = atan2(v(2), v(1));
    
    % Get rotation matrix
    R = @(a)( [cos(a) -sin(a); sin(a) cos(a)]);
    
    % Length of vector
    length = 0.2;
    length = norm(v)
    %length = 70;

    
    
    
    % If ca 0, do not plot arrow
    if norm(v) < 0.01
        handle = [];
        return;
    end
    
    % Plot the actual line
    handle = plotVector(pos, length*v/norm(v), style);
    
    % Calculate the vector
    vector = R(angle) * [length; 0];           
    stopPos = pos + vector;

   
    % Plot the arrowhead to Stop          
    target = R(angle+pi/2+pi)*scale*arrow;
    handle(end+1) = plot(stopPos(1)+target(1,:),stopPos(2)+target(2,:),'k');
    handle(end+1) = patch(stopPos(1)+target(1,:), stopPos(2)+target(2,:), 'k');
    
    % if plot text
    if nargin > 3
       tx = mean([pos(1) stopPos(1)]);
       ty = mean([pos(2) stopPos(2)]);
       
       % Augment a bit
       %tx = tx + 0.05*sign(tx);
       %ty = ty + 0.03*sign(ty);
       
       handle(end+1) = text(tx, ty, theText);
    end

end




